/**
 * Pretty printer for expressions
 */

package expression;

import java.io.*;
import expression.Tree.*;

/** pretty printer visitor
 */
public class Printer implements Tree.Visitor {

    PrintStream p;
        
    /** the main printer method
     */
    public static void print(PrintStream p, Tree tree) {
	tree.apply(new Printer(p));
    }
    

    /** the print method for recursive calls
     *  this is the reuse version of the visitor
     */
    protected void print(Tree tree) {
	tree.apply(this);
    }
    
    /** construct a new printer
     */
    Printer(PrintStream p) {
	this.p = p;
    }
    
    /** the visitor methods
     */
    public void caseProgram(Program tree) {
	for (int i = 0; i < tree.defs.length; i ++) {
	    print(tree.defs[i]);
	    nl();
	}
    }
    
    public void caseDef(Def tree) {
	// fill
    }
        
    public void caseIdent(Ident tree) {
	// fill
    }
    
    public void caseIntLiteral(IntLiteral tree) {
	p.print("" + tree.i);
    }
    
    public void caseBinOp(BinOp tree) {
	// fill
    }
        
    public void caseBad(Bad tree) {
	p.print("???");
    }

    /** indent, indent(), undent(), nl()
	are useful for indenting code.
	They are not really useful in this example
    */
    int indent = 0;
    
    void indent() {
	indent ++;
    }
    
    void undent() {
	indent --;
    }
    
    void nl() {
	p.println();
	for (int i = 0; i < indent; i ++) {
	    p.print("   ");
	}
    }

}
